package scripts;

import jinro.*;
import scripts.roles.*;
import java.text.Collator;

// Game class must inherit this class.

abstract class System {
	def strings = JinroLocalization.getBundle("System.properties");
	// List of Actors
	def actors = [];
	// Dead's history
	def history = [];
	// First victim
	// This property is set by JinroGame.setFirstVictim(GroovyObject actor).
	def firstvictim;
	// Name of times
	// This property is referred by JinroGame.addLog(String message) etc.
	def timename = [strings.getString("time.daytime"), strings.getString("time.nighttime")];
	// Returns all state names.
	// This method is called by JinroGame.getStateNames().
	def getStateNames() {
		def list = [];
		State.each { list << it.name };
		return list;
	}
	// Returns the proper role class for the role's name.
	// This method is called by JinroGame.getRole(String name).
	def getRole(name) {
		return roles.find { it.newInstance().getRoleName() == name };
	}
	// Returns the proper state object for the state's name.
	// This method is called by JinroGame.getState(String name).
	def getState(name) {
		return State.find { it.name == name };
	}
	// Adds an actor.
	// This method is called by JinroGame.addActor(String name, String skypeid).
	def addActor(name, skypeid = null) {
		if(actors.find { it.name == name }) {
			return false;
		}
		Actor newactor = new Actor(name);
		if(skypeid) {
			if(actors.find { it.skypeid == skypeid }) {
				return false;
			}
			newactor.skypeid = skypeid;
		}
		actors << newactor;
		noticeall(strings.getString("entry.added", newactor.name));
		return true;
	}
	// Removes an actor.
	// This method is called by JinroGame.removeActor(String name)
	def removeActor(name) {
		def actor = actors.find { it.name == name };
		actors.remove(actor);
		noticeall(strings.getString("entry.removed", actor.name));
	}
	// Returns the proper actor object for the actor's name.
	// This method is called by JinroGame.getActor(String name)
	def getActor(name) {
		return actors.find { it.name == name };
	}
	// Checks if the name already exists as an actor's.
	// This method is called by JinroGame.checkNameDuplication(String str).
	def checkNameDuplication(name) {
		return !actors.find { it.name == name };
	}
	// Returns the proper name for the Skype ID.
	// This method is called by JinroGame.getNamebySkypeid(String skypeid).
	def getNamebySkypeid(skypeid) {
		return actors.find { it.skypeid == skypeid }?.name;
	}
	// Deals the role to each of the actors.
	// This method is called by JinroGame.dealRoles().
	def dealRoles() {
		def deals = [];
		def roles = JinroGame.getRoles();
		roles.findAll{ !it.is(Civilian) }.each { role ->
			role.number.times { deals << role };
		};
		(actors.size() - deals.size()).times { deals << Civilian };
		def deals2 = deals.findAll { !it.bite };
		def choice = deals2.get(Math.floor(Math.random() * deals2.size()) as int);
		deals.remove(choice);
		firstvictim.setRole(choice);
		firstvictim.killed(State.bitten);
		actors.findAll{ !it.is(firstvictim) }.each {
			it.setRole(deals.remove(Math.floor(Math.random() * deals.size()) as int));
			it.state = State.live;
			it.nextstate = State.live;
			it.countdown = 0;
		};
		JinroGame.day = 0;
		JinroGame.time = 1;
		JinroGame.onPhase = false;
		history.clear();
		// Notifies all actors of each of their names.
		def sortedactors = actors.clone();
		def collator = Collator.getInstance(JinroLocalization.getLocale());
		collator.setStrength(Collator.PRIMARY);
		sortedactors.sort { one, two -> collator.compare(one.name, two.name) };
		noticeall(strings.getString("notice.entries.start"));
		def clipboard = new StringBuilder(10 * sortedactors.size());
		clipboard << strings.getString("notice.entries.start") << "\n";
		sortedactors.each {
			noticeall(it.name);
			clipboard << it.name << "\n";
		}
		noticeall(strings.getString("notice.entries.end", sortedactors.size()));
		clipboard << strings.getString("notice.entries.end", sortedactors.size());
		JinroInterface.copyClipboard(clipboard.toString().denormalize());
		noticeall(strings.getString("dealt"));
	}
	// Add to the dead's history.
	// This method is called by JinroGame.addHistory(Object actor).
	def addHistory(actor) {
		history << actor;
	}
	// Notifies the each of actors of his/her role.
	// This method is called by JinroInterface.makeDialogStartGame().
	def noticeRoles() {
		actors.each { it.noticeRole() };
		makeChats();
	}
	// Makes group chats.
	def makeChats() {
		// For Werewolves
		def wolves = actors.findAll { it.isWolf() && it.skypeid };
		if(wolves.size() > 1) {
			def skypeids = [];
			wolves.each { skypeids << it.skypeid };
			JinroSkype.notice(skypeids as String[], strings.getString("chat.created", JinroGame.getAlias((new Wolf()).getRoleName())));
		}
		// For Sharers
		def sharers = actors.findAll { it.roleIs(new Sharer()) && it.skypeid };
		if(sharers.size() > 1) {
			def skypeids = [];
			sharers.each { skypeids << it.skypeid };
			JinroSkype.notice(skypeids as String[], strings.getString("chat.created", JinroGame.getAlias((new Sharer()).getRoleName())));
		}
	}
	// Sends the command for voting or any actions.
	// This method is called by JinroGame.sendMessage(String name, String type, String target).
	def sendMessage(name, type, target) {
		def actor = actors.find { it.name == name };
		def targetactors = target ? actors.findAll { it.name.contains(target) } : [];
		targetactors.sort { one, two -> one.name.size() <=> two.name.size() };
		if(actor?.isLive()) {
			if(target && targetactors[0] && targetactors[0].name != target) {
				actor.notice(strings.getString("confirm.commandtarget", type, targetactors[0].name));
				actor.suspended = [ type: type, target: targetactors[0] ];
			} else if(!target && actor.suspended?.type == type) {
				actor.receiveMessage(type, actor.suspended.target);
				actor.suspended = null;
			} else {
				actor.receiveMessage(type, targetactors[0]);
				actor.suspended = null;
			}
		}
	}
	// Notifies all actors of the message.
	def noticeall(message) {
		JinroGame.addLog(strings.getString("notice.log", message));
		JinroSkype.noticeAll(message);
	}
}
